from pyclearpass.common import (
    _generate_parameterised_url,
    _remove_empty_keys,
    ClearPassAPILogin,
)

# API Category Name: SessionControl
# FileName: api_sessioncontrol.py


class ApiSessionControl(ClearPassAPILogin):

    # API Service: Manage active sessions
    def get_session(self, filter="", sort="", offset="", limit="", calculate_count=""):
        """
        Operation: Get a list of active sessions
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default -id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/session"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def get_session_by_id(self, id=""):
        """
        Operation: Get an active session
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: URL parameter id
        """
        url_path = "/session/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    # API Service: Disconnect an active session
    def new_session_by_id_disconnect(self, id="", body=({})):
        """
        Operation: Disconnect active session
        HTTP Response Codes: 200 OK, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: id, Description: ID of the session to disconnect
        Required Body Parameters:['id', 'confirm_disconnect']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : "", #ID of the session to disconnect. Object Type: string
        "confirm_disconnect" : False, #Flag to confirm disconnecting the active session. Object Type: boolean

        }
        """
        url_path = "/session/{id}/disconnect"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    # API Service: Reauthorize an active session
    def get_session_by_id_reauthorize(self, template_type="", id=""):
        """
        Operation: Get reauthorization profile(s) for an active session
        HTTP Response Codes: 200 OK, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type (Optional): query, Name: template_type, Description: Reauthorize profile type (Disconnect/CoA)
        Parameter Type: path, Name: id, Description: ID of the session to reauthorize
        """
        url_path = "/session/{id}/reauthorize"
        dict_query = {"template_type": template_type}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_session_by_id_reauthorize(self, id="", body=({})):
        """
        Operation: Reauthorize active session
        HTTP Response Codes: 200 OK, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: id, Description: ID of the session to reauthorize
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "confirm_reauthorize" : False, #Flag to confirm the session reauthorization. Object Type: boolean
        "reauthorize_profile" : "", #Specify the name of the reauthorization profile to apply to the session. Object Type: string

        }
        """
        url_path = "/session/{id}/reauthorize"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    # API Service: Manage Session Action Methods
    def new_session_action_disconnect(self, async_="", body=({})):
        """
        Operation: Performs active session disconnect with a synchronous or asynchronous action & filter by multiple fields.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/disconnect"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_disconnect_mac_by_mac_address(
        self, async_="", mac_address="", body=({})
    ):
        """
        Operation: Performs active session disconnect with a synchronous or asynchronous action & filter by mac-address.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: mac_address, Description: filter by mac-address
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/disconnect/mac/{mac_address}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"mac_address": mac_address}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_disconnect_username_by_username(
        self, async_="", username="", body=({})
    ):
        """
        Operation: Performs active session disconnect with a synchronous or asynchronous action & filter by username.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: username, Description: filter by username
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/disconnect/username/{username}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"username": username}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_disconnect_ip_by_client_ip_address(
        self, async_="", client_ip_address="", body=({})
    ):
        """
        Operation: Performs active session disconnect with a synchronous or asynchronous action & filter by client-ip-address.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: client_ip_address, Description: filter by client-ip-address
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/disconnect/ip/{client_ip_address}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"client_ip_address": client_ip_address}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_coa(self, async_="", body=({})):
        """
        Operation: Performs active session reauthorize with a synchronous or asynchronous action & filter by multiple fields.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/coa"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_coa_mac_by_mac_address(
        self, async_="", mac_address="", body=({})
    ):
        """
        Operation: Performs active session reauthorize with a synchronous or asynchronous action & filter by mac-address.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: mac_address, Description: filter by mac-address
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/coa/mac/{mac_address}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"mac_address": mac_address}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_coa_username_by_username(
        self, async_="", username="", body=({})
    ):
        """
        Operation: Performs active session reauthorize with a synchronous or asynchronous action & filter by username.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: username, Description: filter by username
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/coa/username/{username}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"username": username}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def new_session_action_coa_ip_by_client_ip_address(
        self, async_="", client_ip_address="", body=({})
    ):
        """
        Operation: Performs active session reauthorize with a synchronous or asynchronous action & filter by client-ip-address.
        HTTP Response Codes: 200 OK, 202 Accepted, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: async, Description: trigger asynchronous API calls
        Parameter Type: path, Name: client_ip_address, Description: filter by client-ip-address
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "filter" : {}, #JSON filter expression specifying the items to return, e.g. "filter":{"username":"admin", ...}. Object Type: object
        "enforcement_profile" : {}, #When performing CoA, the 'enforcement_profile' parameter MUST be provided, e.g. "enforcement_profile":["...", "...'"]. Object Type: object
        "sort" : "", #Sorting the result set, e.g. "sort": "-id". Object Type: string

        }
        """
        url_path = "/session-action/coa/ip/{client_ip_address}"
        dict_query = {"async_": async_}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"client_ip_address": client_ip_address}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_session_action_by_action_id(self, offset="", limit="", action_id=""):
        """
        Operation: Returns the current status of a disconnected or reauthorized active session by the action ID.
        HTTP Response Codes: 200 OK, 400 Bad Request, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type (Optional): query, Name: offset, Description: Starting point to return rows from a result set. i.e Default: 0
        Parameter Type (Optional): query, Name: limit, Description: Limit the number of rows returned from a result set. i.e Default: 25
        Parameter Type: path, Name: action_id, Description: request body
        """
        url_path = "/session-action/{action_id}"
        dict_query = {"offset": offset, "limit": limit}
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        dict_path = {"action_id": action_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")



from pyclearpass.common import (
    _generate_parameterised_url,
    _remove_empty_keys,
    ClearPassAPILogin,
)

# API Category Name: PlatformCertificates
# FileName: api_platformcertificates.py


class ApiPlatformCertificates(ClearPassAPILogin):

    # API Service: Manage Certificate Signing Requests
    def new_cert_sign_request(self, body=({})):
        """
        Operation: Post a certificate sign request
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters:['subject_CN', 'private_key_password', 'private_key_type', 'digest_algorithm']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "subject_CN" : "", #Common Name (CN). Object Type: string
        "subject_O" : "", #Organization (O). Object Type: string
        "subject_OU" : "", #Organizational Unit (OU). Object Type: string
        "subject_L" : "", #Location (L). Object Type: string
        "subject_ST" : "", #State (ST). Object Type: string
        "subject_C" : "", #Country (C). Object Type: string
        "subject_SAN" : "", #Subject Alternate Name (SAN). Object Type: string
        "private_key_password" : "", #Private Key Password. Object Type: string
        "private_key_type" : "", #null. Object Type: string
        "digest_algorithm" : "", #Digest Algorithm. Object Type: string

        }
        """
        url_path = "/cert-sign-request"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    # API Service: Manage Certificate Trust List
    def get_cert_trust_list(
        self, filter="", sort="", offset="", limit="", calculate_count=""
    ):
        """
        Operation: Get a list of certificate trust list
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default +id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/cert-trust-list"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_cert_trust_list(self, body=({})):
        """
        Operation: Add a new certificate trust list
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters:['cert_file', 'cert_usage']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "cert_file" : "", #Certificate trust list file. Object Type: string
        "enabled" : False, #Enable certificate trust list. Object Type: boolean
        "cert_usage" : False, #Usage of the certificate. Object Type: array

        }
        """
        url_path = "/cert-trust-list"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_cert_trust_list_by_cert_trust_list_id(self, cert_trust_list_id=""):
        """
        Operation: Get a certificate trust list
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: cert_trust_list_id, Description: URL parameter cert_trust_list_id
        """
        url_path = "/cert-trust-list/{cert_trust_list_id}"
        dict_path = {"cert_trust_list_id": cert_trust_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_cert_trust_list_by_cert_trust_list_id(
        self, cert_trust_list_id="", body=({})
    ):
        """
        Operation: Enbale certificate trust list
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: cert_trust_list_id, Description: URL parameter cert_trust_list_id
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "cert_file" : "", #Certificate trust list file. Object Type: string
        "enabled" : False, #Enable certificate trust list. Object Type: boolean
        "cert_usage" : False, #Usage of the certificate. Object Type: array

        }
        """
        url_path = "/cert-trust-list/{cert_trust_list_id}"
        dict_path = {"cert_trust_list_id": cert_trust_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def delete_cert_trust_list_by_cert_trust_list_id(self, cert_trust_list_id=""):
        """
        Operation: Delete a certificate trust list
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: cert_trust_list_id, Description: URL parameter cert_trust_list_id
        """
        url_path = "/cert-trust-list/{cert_trust_list_id}"
        dict_path = {"cert_trust_list_id": cert_trust_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    # API Service: Manage Certificate Trust List details
    def get_cert_trust_list_details(
        self, filter="", sort="", offset="", limit="", calculate_count=""
    ):
        """
        Operation: Get a list of certificate trust list details
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default +id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/cert-trust-list-details"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def get_cert_trust_list_details_by_cert_trust_list_details_id(
        self, cert_trust_list_details_id=""
    ):
        """
        Operation: Get a certificate trust list details
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: cert_trust_list_details_id, Description: URL parameter cert_trust_list_details_id
        """
        url_path = "/cert-trust-list-details/{cert_trust_list_details_id}"
        dict_path = {"cert_trust_list_details_id": cert_trust_list_details_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    # API Service: Manage Client Certificates
    def get_client_cert(
        self, filter="", sort="", offset="", limit="", calculate_count=""
    ):
        """
        Operation: Get a list of client certificates
        HTTP Response Codes: 200 OK, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default +id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/client-cert"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_client_cert(self, body=({})):
        """
        Operation: Add a client certificate
        HTTP Response Codes: 201 Created, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "certificate_url" : "", #Certificate File URL. Object Type: string
        "pkcs12_file_url" : "", #PKCS12 File URL. Object Type: string
        "pkcs12_passphrase" : "", #PKCS12 passphrase. Object Type: string

        }
        """
        url_path = "/client-cert"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_client_cert_by_client_cert_id(self, client_cert_id=""):
        """
        Operation: Get a client certificate
        HTTP Response Codes: 200 OK, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: client_cert_id, Description: URL parameter client_cert_id
        """
        url_path = "/client-cert/{client_cert_id}"
        dict_path = {"client_cert_id": client_cert_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def delete_client_cert_by_client_cert_id(self, client_cert_id=""):
        """
        Operation: Delete a client certificate
        HTTP Response Codes: 204 No Content, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: client_cert_id, Description: URL parameter client_cert_id
        """
        url_path = "/client-cert/{client_cert_id}"
        dict_path = {"client_cert_id": client_cert_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    # API Service: Manage Revocation Certificate List
    def get_revocation_list(
        self, filter="", sort="", offset="", limit="", calculate_count=""
    ):
        """
        Operation: Get a list of revocation certificate list
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default +id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/revocation-list"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_revocation_list(self, body=({})):
        """
        Operation: Add a new revocation certificate list
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters:['url']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "url" : "", #URL of the Certificate. Object Type: string
        "bypass_proxy" : False, #Bypass Proxy status of the Certificate. Object Type: boolean
        "periodic_update" : False, #Periodic Update status of the Certificate. Object Type: boolean
        "periodic_update_interval" : 0, #Periodic update time of the Certificate. Object Type: integer
        "last_update_status" : "", #Last updated status. Object Type: string
        "last_updated_time" : "", #Last updated time. Object Type: string
        "next_updated_time" : "", #Next update time. Object Type: string

        }
        """
        url_path = "/revocation-list"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_revocation_list_by_revocation_list_id(self, revocation_list_id=""):
        """
        Operation: Get a certificate by id
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: revocation_list_id, Description: URL parameter revocation_list_id
        """
        url_path = "/revocation-list/{revocation_list_id}"
        dict_path = {"revocation_list_id": revocation_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_revocation_list_by_revocation_list_id(
        self, revocation_list_id="", body=({})
    ):
        """
        Operation: Update a certificate
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: revocation_list_id, Description: URL parameter revocation_list_id
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "url" : "", #URL of the Certificate. Object Type: string
        "bypass_proxy" : False, #Bypass Proxy status of the Certificate. Object Type: boolean
        "periodic_update" : False, #Periodic Update status of the Certificate. Object Type: boolean
        "periodic_update_interval" : 0, #Periodic update time of the Certificate. Object Type: integer
        "last_update_status" : "", #Last updated status. Object Type: string
        "last_updated_time" : "", #Last updated time. Object Type: string
        "next_updated_time" : "", #Next update time. Object Type: string

        }
        """
        url_path = "/revocation-list/{revocation_list_id}"
        dict_path = {"revocation_list_id": revocation_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def delete_revocation_list_by_revocation_list_id(self, revocation_list_id=""):
        """
        Operation: Delete a certificate list by id
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: revocation_list_id, Description: URL parameter revocation_list_id
        """
        url_path = "/revocation-list/{revocation_list_id}"
        dict_path = {"revocation_list_id": revocation_list_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    # API Service: Manage Self-Signed Certificates
    def new_self_signed_cert(self, body=({})):
        """
        Operation: Create a Self-Signed Certificate
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters:['certificate_type', 'server', 'type', 'subject_CN', 'private_key_password', 'private_key_type', 'digest_algorithm']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "certificate_type" : "", #Certificate Type. Object Type: string
        "server" : "", #Server hostname. Object Type: string
        "type" : "", #Service Type. Object Type: string
        "subject_CN" : "", #Common Name (CN). Object Type: string
        "subject_O" : "", #Organization (O). Object Type: string
        "subject_OU" : "", #Organizational Unit (OU). Object Type: string
        "subject_L" : "", #Location (L). Object Type: string
        "subject_ST" : "", #State (ST). Object Type: string
        "subject_C" : "", #Country (C). Object Type: string
        "subject_SAN" : "", #Subject Alternate Name (SAN). Object Type: string
        "private_key_password" : "", #Private Key Password. Object Type: string
        "private_key_type" : "", #Private Key Type. Object Type: string
        "digest_algorithm" : "", #Digest Algorithm. Object Type: string

        }
        """
        url_path = "/self-signed-cert"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    # API Service: Manage Server Certificates
    def get_server_cert(self):
        """
        Operation: Get a list of server certificates
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type
        """
        url_path = "/server-cert"
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def get_server_cert_by_service_id(self, service_id=""):
        """
        Operation: Get server certificate
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: service_id, Description: Unique ID for the service
        """
        url_path = "/server-cert/{service_id}"
        dict_path = {"service_id": service_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_server_cert_by_service_id(self, service_id="", body=({})):
        """
        Operation: Update a server certificate
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: service_id, Description: URL parameter service_id
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "service_id" : 0, #Service ID of the server certificate(1-RADIUS server cert, 2-HTTPS(ECC) server cert, 7-HTTPS(RSA) server cert,21-RadSec server cert and 106-Database server cert). Object Type: integer
        "service_name" : "", #Service Name of the server certificate(RADIUS, HTTPS(ECC), HTTPS(RSA), RadSec or Database). Object Type: string
        "certificate_type" : "", #Type of the server certificate. Object Type: string
        "subject" : "", #Subject of the server certificate . Object Type: string
        "expiry_date" : "", #Expiry date of the server certificate . Object Type: string
        "issue_date" : "", #Issue date of the server certificate. Object Type: string
        "issue_by" : "", #server certificate issued by. Object Type: string
        "validity" : "", #Validity of the server certificate . Object Type: string
        "public_key_algorithm" : "", #Public Key Algorithm of Certificate.. Object Type: string
        "enabled" : False, #Enabled State of the certificate.. Object Type: boolean
        "root_ca_cert" : {}, #Root CA Certificate. Object Type: object
        "intermediate_ca_cert" : [{"":""}], #Intermediate CA Certificate. Object Type: array
        "cert_file" : "", #Certificate File. Object Type: string
        "certificate_url" : "", #Certificate File URL. Object Type: string
        "pkcs12_file_url" : "", #PKCS12 File URL. Object Type: string
        "pkcs12_passphrase" : "", #PKCS12 passphrase. Object Type: string

        }
        """
        url_path = "/server-cert/{service_id}"
        dict_path = {"service_id": service_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def get_server_cert_name_by_server_uuid_service_name(
        self, server_uuid="", service_name=""
    ):
        """
        Operation: Get a server certificate
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: server_uuid, Description: UUID of the server
        Parameter Type: path, Name: service_name, Description: Service name (RADIUS, HTTPS(RSA), HTTPS(ECC) or RadSec)
        """
        url_path = "/server-cert/name/{server_uuid}/{service_name}"
        dict_path = {"server_uuid": server_uuid, "service_name": service_name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def replace_server_cert_name_by_server_uuid_service_name(
        self, server_uuid="", service_name="", body=({})
    ):
        """
        Operation: Add a server certificate
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: server_uuid, Description: UUID of the server
        Parameter Type: path, Name: service_name, Description: Service name (RADIUS, HTTPS(RSA), HTTPS(ECC) or RadSec)
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "certificate_url" : "", #Certificate File URL. Object Type: string
        "pkcs12_file_url" : "", #PKCS12 File URL. Object Type: string
        "pkcs12_passphrase" : "", #PKCS12 passphrase. Object Type: string

        }
        """
        url_path = "/server-cert/name/{server_uuid}/{service_name}"
        dict_path = {"server_uuid": server_uuid, "service_name": service_name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="put", query=body
        )

    def update_server_cert_name_by_server_uuid_service_name_enable(
        self, server_uuid="", service_name=""
    ):
        """
        Operation: Update a server certificate
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: server_uuid, Description: UUID of the server
        Parameter Type: path, Name: service_name, Description: Service name (HTTPS(RSA) or HTTPS(ECC))
        """
        url_path = "/server-cert/name/{server_uuid}/{service_name}/enable"
        dict_path = {"server_uuid": server_uuid, "service_name": service_name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_server_cert_name_by_server_uuid_service_name_disable(
        self, server_uuid="", service_name=""
    ):
        """
        Operation: Update a server certificate
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type: path, Name: server_uuid, Description: UUID of the server
        Parameter Type: path, Name: service_name, Description: Service name (HTTPS(RSA) or HTTPS(ECC))
        """
        url_path = "/server-cert/name/{server_uuid}/{service_name}/disable"
        dict_path = {"server_uuid": server_uuid, "service_name": service_name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    # API Service: Manage Service Certificates
    def get_service_cert(
        self, filter="", sort="", offset="", limit="", calculate_count=""
    ):
        """
        Operation: Get a list of service certificates
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Parameter Type (Optional): query, Name: filter, Description: JSON filter expression specifying the items to return
        Parameter Type (Optional): query, Name: sort, Description: Sort ordering for returned items (default +id)
        Parameter Type (Optional): query, Name: offset, Description: Zero based offset to start from
        Parameter Type (Optional): query, Name: limit, Description: Maximum number of items to return (1 – 1000)
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/service-cert"
        dict_query = {
            "filter": filter,
            "sort": sort,
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_service_cert(self, body=({})):
        """
        Operation: Add a service certificate
        HTTP Response Codes: 201 Created, 406 Not Acceptable, 415 Unsupported Media Type, 422 Unprocessable Entity
        Required Body Parameters: None listed
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "certificate_url" : "", #Certificate File URL. Object Type: string
        "pkcs12_file_url" : "", #PKCS12 File URL. Object Type: string
        "pkcs12_passphrase" : "", #PKCS12 passphrase. Object Type: string

        }
        """
        url_path = "/service-cert"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_service_cert_by_service_cert_id(self, service_cert_id=""):
        """
        Operation: Get a service certificate
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: service_cert_id, Description: URL parameter service_cert_id
        """
        url_path = "/service-cert/{service_cert_id}"
        dict_path = {"service_cert_id": service_cert_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def delete_service_cert_by_service_cert_id(self, service_cert_id=""):
        """
        Operation: Delete a service certificate
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: service_cert_id, Description: URL parameter service_cert_id
        """
        url_path = "/service-cert/{service_cert_id}"
        dict_path = {"service_cert_id": service_cert_id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")



from pyclearpass.common import (
    _generate_parameterised_url,
    _remove_empty_keys,
    ClearPassAPILogin,
)

# API Category Name: Insight
# FileName: api_insight.py


class ApiInsight(ClearPassAPILogin):

    # API Service: Operations for Alert
    def get_alert(self, offset="", limit="", calculate_count=""):
        """
        Operation: Get all Insight alert configurations.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type (Optional): query, Name: offset, Description: Starting point to return rows from a result set. i.e Default: 0
        Parameter Type (Optional): query, Name: limit, Description: Limit the number of rows returned from a result set. i.e Default: 25
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/alert"
        dict_query = {
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_alert(self, body=({})):
        """
        Operation: Create an Insight alert configuration.
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type
        Required Body Parameters:['id', 'name', 'category', 'subcategory', 'email_targets', 'severity', 'threshold', 'interval', 'interval_unit']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the alert. Object Type: integer
        "name" : "", #Name of the alert. Object Type: string
        "description" : "", #Description of the alert. Object Type: string
        "category" : "", #Category of the alert. Object Type: string
        "subcategory" : "", #Sub category is the template name of the alert. Object Type: string
        "email_targets" : {}, #Send alert notification to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send alert notification to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "config" : {},      #Setting the alert filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     }     #}. Object Type: object
        "severity" : "", #Severity of the alert, either "critical" or "warning". Object Type: string
        "threshold" : 0, #Triggering the alert when reaching the specified numeric threshold value. Object Type: integer
        "interval" : 0, #Triggering the alert at the numeric interval. Object Type: integer
        "interval_unit" : "", #Interval units either "minute" or "hour". Object Type: string
        "is_enabled" : False, #Enable/Disable the alert. Object Type: boolean
        "is_muted" : False, #Mute/Unmute the alert. Object Type: boolean

        }
        """
        url_path = "/alert"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_alert_by_id(self, id=""):
        """
        Operation: Get an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_alert_by_id(self, id="", body=({})):
        """
        Operation: Partial update of an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        Required Body Parameters:['id', 'name', 'category', 'subcategory', 'email_targets', 'severity', 'threshold', 'interval', 'interval_unit']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the alert. Object Type: integer
        "name" : "", #Name of the alert. Object Type: string
        "description" : "", #Description of the alert. Object Type: string
        "category" : "", #Category of the alert. Object Type: string
        "subcategory" : "", #Sub category is the template name of the alert. Object Type: string
        "email_targets" : {}, #Send alert notification to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send alert notification to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "config" : {},      #Setting the alert filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     }     #}. Object Type: object
        "severity" : "", #Severity of the alert, either "critical" or "warning". Object Type: string
        "threshold" : 0, #Triggering the alert when reaching the specified numeric threshold value. Object Type: integer
        "interval" : 0, #Triggering the alert at the numeric interval. Object Type: integer
        "interval_unit" : "", #Interval units either "minute" or "hour". Object Type: string
        "is_enabled" : False, #Enable/Disable the alert. Object Type: boolean
        "is_muted" : False, #Mute/Unmute the alert. Object Type: boolean

        }
        """
        url_path = "/alert/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def replace_alert_by_id(self, id="", body=({})):
        """
        Operation: Complete update of an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        Required Body Parameters:['id', 'name', 'category', 'subcategory', 'email_targets', 'severity', 'threshold', 'interval', 'interval_unit']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the alert. Object Type: integer
        "name" : "", #Name of the alert. Object Type: string
        "description" : "", #Description of the alert. Object Type: string
        "category" : "", #Category of the alert. Object Type: string
        "subcategory" : "", #Sub category is the template name of the alert. Object Type: string
        "email_targets" : {}, #Send alert notification to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send alert notification to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "config" : {},      #Setting the alert filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     }     #}. Object Type: object
        "severity" : "", #Severity of the alert, either "critical" or "warning". Object Type: string
        "threshold" : 0, #Triggering the alert when reaching the specified numeric threshold value. Object Type: integer
        "interval" : 0, #Triggering the alert at the numeric interval. Object Type: integer
        "interval_unit" : "", #Interval units either "minute" or "hour". Object Type: string
        "is_enabled" : False, #Enable/Disable the alert. Object Type: boolean
        "is_muted" : False, #Mute/Unmute the alert. Object Type: boolean

        }
        """
        url_path = "/alert/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="put", query=body
        )

    def delete_alert_by_id(self, id=""):
        """
        Operation: Delete an Insight alert configuration by id.
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    def get_alert_by_name(self, name=""):
        """
        Operation: Get an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_alert_by_name(self, name="", body=({})):
        """
        Operation: Partial update of an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        Required Body Parameters:['id', 'name', 'category', 'subcategory', 'email_targets', 'severity', 'threshold', 'interval', 'interval_unit']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the alert. Object Type: integer
        "name" : "", #Name of the alert. Object Type: string
        "description" : "", #Description of the alert. Object Type: string
        "category" : "", #Category of the alert. Object Type: string
        "subcategory" : "", #Sub category is the template name of the alert. Object Type: string
        "email_targets" : {}, #Send alert notification to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send alert notification to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "config" : {},      #Setting the alert filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     }     #}. Object Type: object
        "severity" : "", #Severity of the alert, either "critical" or "warning". Object Type: string
        "threshold" : 0, #Triggering the alert when reaching the specified numeric threshold value. Object Type: integer
        "interval" : 0, #Triggering the alert at the numeric interval. Object Type: integer
        "interval_unit" : "", #Interval units either "minute" or "hour". Object Type: string
        "is_enabled" : False, #Enable/Disable the alert. Object Type: boolean
        "is_muted" : False, #Mute/Unmute the alert. Object Type: boolean

        }
        """
        url_path = "/alert/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def replace_alert_by_name(self, name="", body=({})):
        """
        Operation: Complete update of an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        Required Body Parameters:['id', 'name', 'category', 'subcategory', 'email_targets', 'severity', 'threshold', 'interval', 'interval_unit']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the alert. Object Type: integer
        "name" : "", #Name of the alert. Object Type: string
        "description" : "", #Description of the alert. Object Type: string
        "category" : "", #Category of the alert. Object Type: string
        "subcategory" : "", #Sub category is the template name of the alert. Object Type: string
        "email_targets" : {}, #Send alert notification to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send alert notification to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "config" : {},      #Setting the alert filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     }     #}. Object Type: object
        "severity" : "", #Severity of the alert, either "critical" or "warning". Object Type: string
        "threshold" : 0, #Triggering the alert when reaching the specified numeric threshold value. Object Type: integer
        "interval" : 0, #Triggering the alert at the numeric interval. Object Type: integer
        "interval_unit" : "", #Interval units either "minute" or "hour". Object Type: string
        "is_enabled" : False, #Enable/Disable the alert. Object Type: boolean
        "is_muted" : False, #Mute/Unmute the alert. Object Type: boolean

        }
        """
        url_path = "/alert/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="put", query=body
        )

    def delete_alert_by_name(self, name=""):
        """
        Operation: Delete an Insight alert configuration by name.
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    def update_alert_by_id_enable(self, id=""):
        """
        Operation: Enable an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}/enable"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_id_disable(self, id=""):
        """
        Operation: Disable an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}/disable"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_id_mute(self, id=""):
        """
        Operation: Mute an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}/mute"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_id_unmute(self, id=""):
        """
        Operation: Unmute an Insight alert configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the alert
        """
        url_path = "/alert/{id}/unmute"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_name_enable(self, name=""):
        """
        Operation: Enable an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}/enable"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_name_disable(self, name=""):
        """
        Operation: Disable an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}/disable"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_name_mute(self, name=""):
        """
        Operation: Mute an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}/mute"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_alert_by_name_unmute(self, name=""):
        """
        Operation: Unmute an Insight alert configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the alert
        """
        url_path = "/alert/{name}/unmute"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    # API Service: Operations for Report
    def get_report(self, offset="", limit="", calculate_count=""):
        """
        Operation: Get all Insight report configurations.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type (Optional): query, Name: offset, Description: Starting point to return rows from a result set. i.e Default: 0
        Parameter Type (Optional): query, Name: limit, Description: Limit the number of rows returned from a result set. i.e Default: 25
        Parameter Type (Optional): query, Name: calculate_count, Description: Whether to calculate the total item count
        """
        url_path = "/report"
        dict_query = {
            "offset": offset,
            "limit": limit,
            "calculate_count": calculate_count,
        }
        url_path = _generate_parameterised_url(parameters=dict_query, url=url_path)
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def new_report(self, body=({})):
        """
        Operation: Create an Insight report configuration.
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 406 Not Acceptable, 415 Unsupported Media Type
        Required Body Parameters:['id', 'name', 'category', 'subcategory']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the report. Object Type: integer
        "name" : "", #Name of the report. Object Type: string
        "description" : "", #Description of the report. Object Type: string
        "language" : "", #Language of the report. Object Type: string
        "category" : "", #Category of the report. Object Type: string
        "subcategory" : "", #Sub category is the template name of the report. Object Type: string
        "email_targets" : {}, #Send report to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send report to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "copy_remote" : False, #Enable to copy the report to the configured SCP/SFTP server. Object Type: boolean
        "config" : {},      #Setting the report filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     },     csv_cols": ["...", "...", "..."]     #}. Object Type: object
        "schedule" : {},      #Scheduling the report. Options are [noRepeat, daily, weekly, monthly],     e.g. when running the report "now" itself => "schedule": {} - then "begin_dt" & "end_dt" are mandatory,     when scheduling the report at "daily" => "schedule": {"freq": "daily", "hour": 12}     when scheduling the report at "weekly" => "schedule": {"freq": "weekly", "day": 0, "hour": 12}     when scheduling the report at "monthly" => "schedule": {"freq": "monthly", "date": 1, "hour": 12}     #. Object Type: object
        "begin_dt" : 0, #Collect the data for the report from this "begin_dt". Object Type: integer
        "end_dt" : "", #Collect the data for the report till this "end_dt". Object Type: string
        "is_enabled" : False, #Enable/Disable the report. Object Type: boolean

        }
        """
        url_path = "/report"
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="post", query=body
        )

    def get_report_by_id(self, id=""):
        """
        Operation: Get an Insight report configuration by id.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        """
        url_path = "/report/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_report_by_id(self, id="", body=({})):
        """
        Operation: Partial update of an Insight report configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        Required Body Parameters:['id', 'name', 'category', 'subcategory']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the report. Object Type: integer
        "name" : "", #Name of the report. Object Type: string
        "description" : "", #Description of the report. Object Type: string
        "language" : "", #Language of the report. Object Type: string
        "category" : "", #Category of the report. Object Type: string
        "subcategory" : "", #Sub category is the template name of the report. Object Type: string
        "email_targets" : {}, #Send report to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send report to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "copy_remote" : False, #Enable to copy the report to the configured SCP/SFTP server. Object Type: boolean
        "config" : {},      #Setting the report filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     },     csv_cols": ["...", "...", "..."]     #}. Object Type: object
        "schedule" : {},      #Scheduling the report. Options are [noRepeat, daily, weekly, monthly],     e.g. when running the report "now" itself => "schedule": {} - then "begin_dt" & "end_dt" are mandatory,     when scheduling the report at "daily" => "schedule": {"freq": "daily", "hour": 12}     when scheduling the report at "weekly" => "schedule": {"freq": "weekly", "day": 0, "hour": 12}     when scheduling the report at "monthly" => "schedule": {"freq": "monthly", "date": 1, "hour": 12}     #. Object Type: object
        "begin_dt" : 0, #Collect the data for the report from this "begin_dt". Object Type: integer
        "end_dt" : "", #Collect the data for the report till this "end_dt". Object Type: string
        "is_enabled" : False, #Enable/Disable the report. Object Type: boolean

        }
        """
        url_path = "/report/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def replace_report_by_id(self, id="", body=({})):
        """
        Operation: Complete update of an Insight report configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        Required Body Parameters:['id', 'name', 'category', 'subcategory']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the report. Object Type: integer
        "name" : "", #Name of the report. Object Type: string
        "description" : "", #Description of the report. Object Type: string
        "language" : "", #Language of the report. Object Type: string
        "category" : "", #Category of the report. Object Type: string
        "subcategory" : "", #Sub category is the template name of the report. Object Type: string
        "email_targets" : {}, #Send report to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send report to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "copy_remote" : False, #Enable to copy the report to the configured SCP/SFTP server. Object Type: boolean
        "config" : {},      #Setting the report filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     },     csv_cols": ["...", "...", "..."]     #}. Object Type: object
        "schedule" : {},      #Scheduling the report. Options are [noRepeat, daily, weekly, monthly],     e.g. when running the report "now" itself => "schedule": {} - then "begin_dt" & "end_dt" are mandatory,     when scheduling the report at "daily" => "schedule": {"freq": "daily", "hour": 12}     when scheduling the report at "weekly" => "schedule": {"freq": "weekly", "day": 0, "hour": 12}     when scheduling the report at "monthly" => "schedule": {"freq": "monthly", "date": 1, "hour": 12}     #. Object Type: object
        "begin_dt" : 0, #Collect the data for the report from this "begin_dt". Object Type: integer
        "end_dt" : "", #Collect the data for the report till this "end_dt". Object Type: string
        "is_enabled" : False, #Enable/Disable the report. Object Type: boolean

        }
        """
        url_path = "/report/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="put", query=body
        )

    def delete_report_by_id(self, id=""):
        """
        Operation: Delete an Insight report configuration by id.
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        """
        url_path = "/report/{id}"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    def get_report_by_name(self, name=""):
        """
        Operation: Get an Insight report configuration by name.
        HTTP Response Codes: 200 OK, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        """
        url_path = "/report/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="get")

    def update_report_by_name(self, name="", body=({})):
        """
        Operation: Partial update of an Insight report configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        Required Body Parameters:['id', 'name', 'category', 'subcategory']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the report. Object Type: integer
        "name" : "", #Name of the report. Object Type: string
        "description" : "", #Description of the report. Object Type: string
        "language" : "", #Language of the report. Object Type: string
        "category" : "", #Category of the report. Object Type: string
        "subcategory" : "", #Sub category is the template name of the report. Object Type: string
        "email_targets" : {}, #Send report to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send report to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "copy_remote" : False, #Enable to copy the report to the configured SCP/SFTP server. Object Type: boolean
        "config" : {},      #Setting the report filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     },     csv_cols": ["...", "...", "..."]     #}. Object Type: object
        "schedule" : {},      #Scheduling the report. Options are [noRepeat, daily, weekly, monthly],     e.g. when running the report "now" itself => "schedule": {} - then "begin_dt" & "end_dt" are mandatory,     when scheduling the report at "daily" => "schedule": {"freq": "daily", "hour": 12}     when scheduling the report at "weekly" => "schedule": {"freq": "weekly", "day": 0, "hour": 12}     when scheduling the report at "monthly" => "schedule": {"freq": "monthly", "date": 1, "hour": 12}     #. Object Type: object
        "begin_dt" : 0, #Collect the data for the report from this "begin_dt". Object Type: integer
        "end_dt" : "", #Collect the data for the report till this "end_dt". Object Type: string
        "is_enabled" : False, #Enable/Disable the report. Object Type: boolean

        }
        """
        url_path = "/report/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="patch", query=body
        )

    def replace_report_by_name(self, name="", body=({})):
        """
        Operation: Complete update of an Insight report configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        Required Body Parameters:['id', 'name', 'category', 'subcategory']
        Parameter Type: body, Name: body
        Body example with descriptions and object types below (type(dict):

        body={
        "id" : 0, #Numeric id of the report. Object Type: integer
        "name" : "", #Name of the report. Object Type: string
        "description" : "", #Description of the report. Object Type: string
        "language" : "", #Language of the report. Object Type: string
        "category" : "", #Category of the report. Object Type: string
        "subcategory" : "", #Sub category is the template name of the report. Object Type: string
        "email_targets" : {}, #Send report to the configured email targets, e.g. "email_targets":["...", "..."]. Object Type: object
        "sms_targets" : {}, #Send report to the configured SMS targets, e.g. "sms_targets":["...", "..."]. Object Type: object
        "copy_remote" : False, #Enable to copy the report to the configured SCP/SFTP server. Object Type: boolean
        "config" : {},      #Setting the report filter configurations & adding CSV columns for the CSV report,     e.g. "config": {     "filter": {     "auth.ap_name": {     "operator": "EQUALS",     "value": ["..."]     },     "cppm_cluster.hostname": {     "operator":"CONTAINS",     "value":["...", "..."]     }     },     csv_cols": ["...", "...", "..."]     #}. Object Type: object
        "schedule" : {},      #Scheduling the report. Options are [noRepeat, daily, weekly, monthly],     e.g. when running the report "now" itself => "schedule": {} - then "begin_dt" & "end_dt" are mandatory,     when scheduling the report at "daily" => "schedule": {"freq": "daily", "hour": 12}     when scheduling the report at "weekly" => "schedule": {"freq": "weekly", "day": 0, "hour": 12}     when scheduling the report at "monthly" => "schedule": {"freq": "monthly", "date": 1, "hour": 12}     #. Object Type: object
        "begin_dt" : 0, #Collect the data for the report from this "begin_dt". Object Type: integer
        "end_dt" : "", #Collect the data for the report till this "end_dt". Object Type: string
        "is_enabled" : False, #Enable/Disable the report. Object Type: boolean

        }
        """
        url_path = "/report/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        body = _remove_empty_keys(keys=body)
        return ClearPassAPILogin._send_request(
            self, url=url_path, method="put", query=body
        )

    def delete_report_by_name(self, name=""):
        """
        Operation: Delete an Insight report configuration by name.
        HTTP Response Codes: 204 No Content, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        """
        url_path = "/report/{name}"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="delete")

    def update_report_by_id_enable(self, id=""):
        """
        Operation: Enable an Insight report configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        """
        url_path = "/report/{id}/enable"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_report_by_id_disable(self, id=""):
        """
        Operation: Disable an Insight report configuration by id.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        """
        url_path = "/report/{id}/disable"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_report_by_name_enable(self, name=""):
        """
        Operation: Enable an Insight report configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        """
        url_path = "/report/{name}/enable"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def update_report_by_name_disable(self, name=""):
        """
        Operation: Disable an Insight report configuration by name.
        HTTP Response Codes: 200 OK, 204 No Content, 304 Not Modified, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        """
        url_path = "/report/{name}/disable"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="patch")

    def new_report_by_id_run(self, id=""):
        """
        Operation: Run an Insight report by id.
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: id, Description: Numeric id of the report
        """
        url_path = "/report/{id}/run"
        dict_path = {"id": id}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="post")

    def new_report_by_name_run(self, name=""):
        """
        Operation: Run an Insight report by name.
        HTTP Response Codes: 201 Created, 401 Unauthorized, 403 Forbidden, 404 Not Found, 406 Not Acceptable, 415 Unsupported Media Type
        Parameter Type: path, Name: name, Description: Name of the report
        """
        url_path = "/report/{name}/run"
        dict_path = {"name": name}
        for item in dict_path:
            url_path = url_path.replace("{" + item + "}", dict_path[item])
        return ClearPassAPILogin._send_request(self, url=url_path, method="post")



import json
import requests
import urllib.parse, urllib3

urllib3.disable_warnings()


class ClearPassAPILogin:
    def __init__(
        self,
        granttype="",
        clientid="",
        clientsecret="",
        username="",
        password="",
        server="",
        api_token="",
        verify_ssl=False,
    ):
        """
        This is the class constructor for the ClearPassModule.

        This constructor is required to be created before any modules can be used and must contain the following function arguments:

        Mandatory Parameters:
        server (string): Website for ClearPass services example - https://yourserver.network.local:443/api
        verify_ssl (boolean, optional): default value False. Allows use of an invalid SSL certificate.

        Option 1 Parameters -
        granttype (string) = ['client_credentials' or 'password' or 'refresh_token']: OAuth2 authentication method,client_id (string): Client ID defined in API Clients,
        clientsecret (string, optional): Client secret, required if the API client is not a public client,
        username (string, optional): Username for authentication, required for grant_type "password",
        password (string, optional): Password for authentication, required for grant_type "password",

        Option 2 Parameters-
        api_token = Provide the api_token which is the 'access token'.

        }

        """
        self.granttype = granttype
        self.clientid = clientid
        self.clientsecret = clientsecret
        self.username = username
        self.password = password
        self.server = server
        self.api_token = api_token
        self.verify_ssl = False

    def _send_request(
        self, url, method, query="", content_response_type="application/json"
    ):
        """Sends a request to the ClearPass server
        :query: must contain the json request if model required
        :url: must contain the /url (e.g. /oauth)
        :method: must contain the post or get request type of method
        :content_response_type: by default is set as Application/Json however can be changed by the method if required and functionality exists.
        :api_token optional[]: must contain the api_token for the calls.
        """
        full_url_path = self.server + url

        if len(self.api_token) == 0:
            cred = _new_api_token(self)
            try:
                self.api_token = cred["access_token"]
            except TypeError:
                pass
            except KeyError:
                pass

        if len(self.api_token) != 0:
            header = {
                "Authorization": "Bearer " + self.api_token,
                "accept": content_response_type,
            }
            if method == "post":
                response = requests.post(
                    url=full_url_path,
                    json=query,
                    headers=header,
                    verify=self.verify_ssl,
                )
            if method == "patch":
                response = requests.patch(
                    url=full_url_path,
                    json=query,
                    headers=header,
                    verify=self.verify_ssl,
                )
            if method == "put":
                response = requests.put(
                    url=full_url_path,
                    json=query,
                    headers=header,
                    verify=self.verify_ssl,
                )
            if method == "get":
                response = requests.get(
                    url=full_url_path,
                    json=query,
                    headers=header,
                    verify=self.verify_ssl,
                )
            if method == "delete":
                response = requests.delete(
                    url=full_url_path,
                    json=query,
                    headers=header,
                    verify=self.verify_ssl,
                )
            if method == "":
                print(
                    "method needs to be supplied before sending a request to ClearPass"
                )

            if "json" in content_response_type:                
                try:
                    return json.loads(response.text)
                except json.decoder.JSONDecodeError:
                    return response.text
            else:
                return response.content
        else:
            print("Problem logging into ClearPass")
            return cred


def _new_api_token(self):
    """
    Operation: Obtain an OAuth2 access token for making API calls
    HTTP Status Response Codes: 200 OK, 400 Bad Request, 406 Not Acceptable, 415 Unsupported Media Type, 200 OK, 400 Bad Request, 406 Not Acceptable, 415 Unsupported Media Type
    Required Body Parameters (body description)- TokenEndpoint {grant_type (string) = ['client_credentials' or 'password' or 'refresh_token']: OAuth2 authentication method,client_id (string): Client ID defined in API Clients,client_secret (string, optional): Client secret, required if the API client is not a public client,username (string, optional): Username for authentication, required for grant_type "password",password (string, optional): Password for authentication, required for grant_type "password",scope (string, optional): Scope of the access request,refresh_token (string, optional): Refresh token issued to the client, required for grant_type "refresh_token"}
    Required Body Parameters (type(dict) body example)- {
    "grant_type": "",
    "client_id": "",
    "client_secret": "",
    "username": "",
    "password": "",
    "scope": "",
    "refresh_token": ""
    }
    """
    model = {
        "grant_type": self.granttype,
        "client_id": self.clientid,
        "client_secret": self.clientsecret,
        "username": self.username,
        "password": self.password,
    }

    full_url_path = self.server + "/oauth"
    response = requests.post(url=full_url_path, json=model, verify=self.verify_ssl)

    try:
        response = json.loads(str(response.text))
        return response

    except json.decoder.JSONDecodeError:
        return response


def _remove_empty_keys(keys):
    remove_empty_values_from_dict = []
    for item in keys:
        if keys[item] == "":
            remove_empty_values_from_dict.append(item)
        else:
            # print(item,": ", keys[item])
            pass
    for removal in remove_empty_values_from_dict:
        del keys[removal]
    return keys


def _generate_parameterised_url(url, parameters=""):
    parameters = _remove_empty_keys(keys=parameters)

    if len(parameters) == 0:
        return url
    else:
        encoded_url = urllib.parse.urlencode(parameters)
        final_url = url + "?" + encoded_url
        return final_url
